"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.installVersion = exports.getAvailableVersions = void 0;
const tslib_1 = require("tslib");
const axios = (0, tslib_1.__importStar)(require("axios"));
const crypto = (0, tslib_1.__importStar)(require("crypto"));
const graceful_fs_1 = require("graceful-fs");
const constants_1 = require("./constants");
const types_1 = require("./types");
const utilities_1 = require("./utilities");
/**
 * Fetch all available releases for given {@link Repositorys}.
 * If no repository is given, all {@link Repositorys} are checked.
 * @param repositorys array of {@link Repositorys}.
 * @defaultValue all {@link Repositorys}
 * @param count max versions to fetch for each {@link Repository}
 * @defaultValue 100
 * @returns * resolves with an array of {@link VersionInfo}
 *          * rejects with an {@link Error}
 */
function getAvailableVersions({ repositorys = [
    types_1.Repositorys.WINEGE,
    types_1.Repositorys.PROTONGE,
    types_1.Repositorys.PROTON,
    types_1.Repositorys.WINELUTRIS
], count = 100 }) {
    var repositorys_1, repositorys_1_1;
    var e_1, _a;
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        const releases = [];
        try {
            for (repositorys_1 = (0, tslib_1.__asyncValues)(repositorys); repositorys_1_1 = yield repositorys_1.next(), !repositorys_1_1.done;) {
                const repo = repositorys_1_1.value;
                switch (repo) {
                    case types_1.Repositorys.WINEGE: {
                        yield (0, utilities_1.fetchReleases)({
                            url: constants_1.WINEGE_URL,
                            type: 'Wine-GE',
                            count: count
                        })
                            .then((fetchedReleases) => {
                            releases.push(...fetchedReleases);
                        })
                            .catch((error) => {
                            throw error;
                        });
                        break;
                    }
                    case types_1.Repositorys.PROTONGE: {
                        yield (0, utilities_1.fetchReleases)({
                            url: constants_1.PROTONGE_URL,
                            type: 'Proton-GE',
                            count: count
                        })
                            .then((fetchedReleases) => {
                            releases.push(...fetchedReleases);
                        })
                            .catch((error) => {
                            throw error;
                        });
                        break;
                    }
                    case types_1.Repositorys.PROTON: {
                        yield (0, utilities_1.fetchReleases)({
                            url: constants_1.PROTON_URL,
                            type: 'Proton',
                            count: count
                        })
                            .then((fetchedReleases) => {
                            releases.push(...fetchedReleases);
                        })
                            .catch((error) => {
                            throw error;
                        });
                        break;
                    }
                    case types_1.Repositorys.WINELUTRIS: {
                        yield (0, utilities_1.fetchReleases)({
                            url: constants_1.WINELUTRIS_URL,
                            type: 'Wine-Lutris',
                            count: count
                        })
                            .then((fetchedReleases) => {
                            releases.push(...fetchedReleases);
                        })
                            .catch((error) => {
                            throw error;
                        });
                        break;
                    }
                    default: {
                        console.warn(`Unknown and not supported repository key passed! Skip fetch for ${repo}`);
                        break;
                    }
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (repositorys_1_1 && !repositorys_1_1.done && (_a = repositorys_1.return)) yield _a.call(repositorys_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return releases;
    });
}
exports.getAvailableVersions = getAvailableVersions;
/**
 * Installs a given version to the given installation directory.
 *
 * @param versionInfo the version to install as {@link VersionInfo}
 * @param installDir absolute path to installation directory
 * @param overwrite allow overwriting existing version installation
 * @defaultValue false
 * @param onProgress callback to get installation progress
 * @defaultValue void
 * @returns * resolves with updated {@link VersionInfo} and the full installation
 *            directory
 *          * rejects with an {@link Error}
 */
function installVersion({ versionInfo, installDir, overwrite = false, onProgress = () => {
    return;
}, abortSignal }) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        /*
         * VARIABLE DECLARATION
         */
        const tarFile = installDir + '/' + versionInfo.download.split('/').slice(-1)[0];
        const installSubDir = installDir + '/' + versionInfo.version;
        const sourceChecksum = versionInfo.checksum
            ? (yield axios.default.get(versionInfo.checksum, {
                responseType: 'text'
            })).data
            : undefined;
        const abortHandler = () => {
            const error = new Error(`Installation of ${versionInfo.version} was aborted!`);
            error.name = 'AbortError';
            (0, utilities_1.unlinkFile)(tarFile);
            if ((0, graceful_fs_1.existsSync)(installSubDir)) {
                (0, graceful_fs_1.rmSync)(installSubDir, { recursive: true });
            }
            throw error;
        };
        /*
         * INSTALLATION PROCESS
         */
        // Check if installDir exist
        if (!(0, graceful_fs_1.existsSync)(installDir)) {
            throw new Error(`Installation directory ${installDir} does not exist!`);
        }
        else if (!(0, graceful_fs_1.statSync)(installDir).isDirectory()) {
            throw new Error(`Installation directory ${installDir} is not a directory!`);
        }
        if (!versionInfo.download) {
            // check versionInfo has download
            throw new Error(`No download link provided for ${versionInfo.version}!`);
        }
        // Check if it already exist
        if ((0, graceful_fs_1.existsSync)(installSubDir) && !overwrite) {
            console.warn(`${versionInfo.version} is already installed. Skip installing! \n
      Consider using 'override: true if you wan't to override it!'`);
            // resolve with disksize
            versionInfo.disksize = (0, utilities_1.getFolderSize)(installSubDir);
            return { versionInfo: versionInfo, installDir: installSubDir };
        }
        // remove tarFile if still exist
        (0, utilities_1.unlinkFile)(tarFile);
        // Download
        yield (0, utilities_1.downloadFile)({
            url: versionInfo.download,
            downloadDir: installDir,
            downsize: versionInfo.downsize,
            onProgress: onProgress,
            abortSignal: abortSignal
        }).catch((error) => {
            if (error.includes('AbortError')) {
                abortHandler();
            }
            (0, utilities_1.unlinkFile)(tarFile);
            throw new Error(`Download of ${versionInfo.version} failed with:\n ${error}`);
        });
        // Check if download checksum is correct
        const fileBuffer = (0, graceful_fs_1.readFileSync)(tarFile);
        const hashSum = crypto.createHash('sha512');
        hashSum.update(fileBuffer);
        const downloadChecksum = hashSum.digest('hex');
        if (sourceChecksum) {
            if (!sourceChecksum.includes(downloadChecksum)) {
                (0, utilities_1.unlinkFile)(tarFile);
                throw new Error('Checksum verification failed');
            }
        }
        else {
            console.warn(`No checksum provided. Download of ${versionInfo.version} could be invalid!`);
        }
        // Unzip
        try {
            (0, graceful_fs_1.mkdirSync)(installSubDir);
        }
        catch (error) {
            (0, utilities_1.unlinkFile)(tarFile);
            throw new Error(`Failed to make folder ${installSubDir} with:\n ${error}`);
        }
        yield (0, utilities_1.unzipFile)({
            filePath: tarFile,
            unzipDir: installSubDir,
            overwrite: overwrite,
            onProgress: onProgress,
            abortSignal: abortSignal
        }).catch((error) => {
            if (error.includes('AbortError')) {
                abortHandler();
            }
            (0, graceful_fs_1.rmSync)(installSubDir, { recursive: true });
            (0, utilities_1.unlinkFile)(tarFile);
            throw new Error(`Unzip of ${tarFile.split('/').slice(-1)[0]} failed with:\n ${error}`);
        });
        // clean up
        (0, utilities_1.unlinkFile)(tarFile);
        // resolve with disksize
        versionInfo.disksize = (0, utilities_1.getFolderSize)(installSubDir);
        return { versionInfo: versionInfo, installDir: installSubDir };
    });
}
exports.installVersion = installVersion;
//# sourceMappingURL=main.js.map