"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unzipFile = exports.downloadFile = exports.getFolderSize = exports.unlinkFile = exports.fetchReleases = void 0;
const tslib_1 = require("tslib");
const axios = (0, tslib_1.__importStar)(require("axios"));
const graceful_fs_1 = require("graceful-fs");
const child_process_1 = require("child_process");
/**
 * Helper to fetch releases from given url.
 *
 * @param url url where to fetch releases from.
 * @param type type of the releases (wine, proton, ge, ...)
 * @param count number of releases to fetch
 * @returns * resolves with an array of {@link VersionInfo}
 *          * rejects with an {@link Error}
 */
function fetchReleases({ url, type, count }) {
    const releases = [];
    return new Promise((resolve, reject) => {
        axios.default
            .get(url + '?per_page=' + count)
            .then((data) => {
            for (const release of data.data) {
                const release_data = {};
                release_data.version = type.includes('Wine')
                    ? `Wine-${release.tag_name}`
                    : `Proton-${release.tag_name}`;
                release_data.type = type;
                release_data.date = release.published_at.split('T')[0];
                release_data.disksize = 0;
                for (const asset of release.assets) {
                    if (asset.name.endsWith('sha512sum')) {
                        release_data.checksum = asset.browser_download_url;
                    }
                    else if (asset.name.endsWith('tar.gz') ||
                        asset.name.endsWith('tar.xz')) {
                        release_data.download = asset.browser_download_url;
                        release_data.downsize = asset.size;
                    }
                }
                releases.push(release_data);
            }
            resolve(releases);
        })
            .catch((error) => {
            reject(new Error(`Could not fetch available releases from ${url} with error:\n ${error}`));
        });
    });
}
exports.fetchReleases = fetchReleases;
/**
 * Helper to unlink a file.
 *
 * @param filePath absolute path to file
 * @throws {@link Error}
 */
function unlinkFile(filePath) {
    try {
        if ((0, graceful_fs_1.existsSync)(filePath)) {
            (0, graceful_fs_1.unlinkSync)(filePath);
        }
    }
    catch (_a) {
        throw new Error(`Couldn't remove ${filePath}!`);
    }
}
exports.unlinkFile = unlinkFile;
/**
 * Helper to get disk space of installed version.
 *
 * @param folder absolute path to folder
 * @returns size of folder in bytes
 */
function getFolderSize(folder) {
    const { stdout } = (0, child_process_1.spawnSync)('du', ['-sb', folder]);
    return parseInt(stdout.toString());
}
exports.getFolderSize = getFolderSize;
/**
 * Helper to download a file via curl.
 *
 * @param url url of the file
 * @param downloadDir absolute path to the download directory
 * @param downsize needed to calculate download speed
 * @param onProgress callback to get download progress
 * @param abortSignal signal to abort spawn
 * @returns resolves or rejects with a message
 */
function downloadFile({ url, downloadDir, downsize, onProgress, abortSignal }) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            try {
                if (!(0, graceful_fs_1.existsSync)(downloadDir)) {
                    reject(`Download path ${downloadDir} does not exist!`);
                }
                else if (!(0, graceful_fs_1.statSync)(downloadDir).isDirectory()) {
                    reject(`Download path ${downloadDir} is not a directory!`);
                }
            }
            catch (error) {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore: can't give error a type but it mostly a Error or SystemError
                reject(error.message);
            }
            let percentage = 0;
            const filePath = downloadDir + '/' + url.split('/').slice(-1)[0];
            const download = (0, child_process_1.spawn)('curl', ['-L', url, '-o', filePath, '-#'], {
                signal: abortSignal
            });
            const startTime = process.hrtime.bigint();
            // curl does somehow print on stderr
            // progress calculation is done on stderr
            download.stderr.on('data', function (stderr) {
                // get time
                const time = process.hrtime.bigint();
                // get info from curl output
                const newPercentage = parseInt(stderr.toString());
                // check if percentage is valid
                percentage =
                    !isNaN(newPercentage) && newPercentage > percentage
                        ? newPercentage
                        : percentage;
                // calculate download speed
                const alreadyDonwloaded = (downsize * percentage) / 100;
                const seconds = Number(time - startTime) / Math.pow(10, 9);
                const avgSpeed = alreadyDonwloaded / seconds;
                // calculate eta
                const eta = percentage > 0 ? (100 * seconds) / percentage - seconds : seconds;
                // Calculate avgSpeed
                onProgress('downloading', {
                    percentage: percentage,
                    avgSpeed: avgSpeed,
                    eta: Math.ceil(eta)
                });
            });
            download.on('close', function (exitcode) {
                onProgress('idle');
                if (exitcode !== 0) {
                    reject(`Download of ${url} failed with exit code:\n ${exitcode}!`);
                }
                resolve(`Succesfully downloaded ${url} to ${filePath}.`);
            });
            download.on('error', (error) => {
                if (error.name.includes('AbortError')) {
                    reject(error.name);
                }
                else {
                    reject(`Download of ${url} failed with:\n ${error.message}!`);
                }
            });
        });
    });
}
exports.downloadFile = downloadFile;
/**
 * Helper to unzip an archive via tar.
 *
 * @param filePath url of the file
 * @param unzipDir absolute path to the unzip directory
 * @param overwrite allow overwriting existing unpacked files
 * @defaultValue false
 * @param onProgress callback to get unzip progress
 * @returns resolves or rejects with a message
 */
function unzipFile({ filePath, unzipDir, overwrite = false, onProgress, abortSignal }) {
    return (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            try {
                if (!(0, graceful_fs_1.existsSync)(filePath)) {
                    reject(`Zip file ${filePath} does not exist!`);
                }
                else if ((0, graceful_fs_1.statSync)(filePath).isDirectory()) {
                    reject(`Archive path ${filePath} is not a file!`);
                }
                else if (!(0, graceful_fs_1.existsSync)(unzipDir)) {
                    reject(`Install path ${unzipDir} does not exist!`);
                }
            }
            catch (error) {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore: can't give error a type but it mostly a Error or SystemError
                reject(error.message);
            }
            let extension_options = '';
            if (filePath.endsWith('tar.gz')) {
                extension_options = '-vzxf';
            }
            else if (filePath.endsWith('tar.xz')) {
                extension_options = '-vJxf';
            }
            else {
                reject(`Archive type ${filePath.split('.').pop()} not supported!`);
            }
            const args = [
                '--directory',
                unzipDir,
                '--strip-components=1',
                extension_options,
                filePath
            ];
            if (overwrite) {
                args.push('--overwrite');
            }
            const unzip = (0, child_process_1.spawn)('tar', args, { signal: abortSignal });
            onProgress('unzipping');
            unzip.stdout.on('data', function () {
                onProgress('unzipping');
            });
            unzip.stderr.on('data', function (stderr) {
                onProgress('idle');
                reject(`Unzip of ${filePath} failed with:\n ${stderr}!`);
            });
            unzip.on('close', function (exitcode) {
                onProgress('idle');
                if (exitcode !== 0) {
                    reject(`Unzip of ${filePath} failed with exit code:\n ${exitcode}!`);
                }
                resolve(`Succesfully unzip ${filePath} to ${unzipDir}.`);
            });
            unzip.on('error', (error) => {
                if (error.name.includes('AbortError')) {
                    reject(error.name);
                }
                else {
                    reject(`Unzip of ${filePath} failed with:\n ${error.message}!`);
                }
            });
        });
    });
}
exports.unzipFile = unzipFile;
//# sourceMappingURL=utilities.js.map